/**
 * Abstract action class.
 * className: action
 *
 * @author Cedric Chantepie (cchantepie@corsaire.fr)
 */
// --- Constructors ---

/**
 * Sole constructor.
 */
function Action() {
  var mainWin = this.mainWindow;

  // init session
  if ((typeof mainWin.session) == "undefined" ||
      mainWin.session == null) {

    mainWin.session = {};
  } // end of if
} // end of <init>

Action.prototype = {
  // --- nsISupports implementation ---

  /**
   * Retrieve instance as specified interface if supported.
   */
  QueryInterface: function QueryInterface(aIID) {
    if (Components.interfaces.nsIClassInfo.equals(aIID) ||
        Components.interfaces.nsISupportsString.equals(aIID) ||
        Components.interfaces.nsISupportsWeakReference.equals(aIID) ||
        Components.interfaces.nsISupports.equals(aIID)) {
      
      return this;
    } // end of if
    
    else {
      throw 0x80004002; // Components.results.NS_NOINTERFACE;
    } // end of else
  }, // end of QueryInterface

  // --- nsIClassInfo implementation ---

  /**
   * Return count of implemented interfaces and 
   * there implemented interfaces.
   *
   * @param count (out) Count of implemented interfaces.
   * @return Array of implemented interfaces. 
   */
  getInterfaces: function getInterfaces(count) {
    count.value = 4;

    return [Components.interfaces.nsIClassInfo,
            Components.interfaces.nsISupportsString,
            Components.interfaces.nsISupportsWeakReference,
            Components.interfaces.nsISupports];

  }, // end of getInterfaces

  /**
   */
  getHelperForLanguage: function getHelperForLanguage(language) { 
    return null;
  }, // end of getHelperForLanguage

  /**
   */
  get contractID() { 
    return null; 
  }, // end of contractID getter

  /**
   * Return description of this class.
   */
  get classDescription() { 
    return "Action"; 
  }, // end of classDescription

  /**
   */
  get classID() {
    return null; 
  }, // end of classID getter

  /**
   * Returns language in which is implemented this class.
   */
  get implementationLanguage() { 
    return Components.interfaces.nsIProgrammingLanguage.JAVASCRIPT; 
  }, // end of implementationLanguage getter
  
  /**
   * Returns class flags.
   */
  get flags() {
    return Components.interfaces.nsIClassInfo.MAIN_THREAD_ONLY;
  }, // end of flags getter

  // --- nsISupportsString implementation ---

  /**
   * Returns string data about this instance.
   */
  get data() {
    return "[object " + this.classDescription + "]"; 
  }, // end of data getter

  /**
   * Returns a string representation of this instance.
   */
  toString: function toString() { 
    return this.data;
  }, // end of toString

  // --- Properties ---

  /**
   * XUL document for controlled view.
   */
  viewDoc: null,
  
  /**
   * Action form.
   */
  form: null,
  
  /**
   * Commands to be processed.
   */
  pendingCommands: {},

  // --- Getters ---

  /**
   * Returns DOM root element.
   */
  get rootElement() {
    var rootElmt = null;
    
    if (this.viewDoc.getElementsByTagName("window").length == 1) {
      return this.viewDoc.getElementsByTagName("window")[0];
    } // end of if
    
    else if (this.viewDoc.getElementsByTagName("dialog").length == 1) {
      return this.viewDoc.getElementsByTagName("dialog")[0];
    } // end of if
    
    else {
      alert("{iPage} [Action] getRootElement -> Unexpected root element");
      
      return null;
    } // end of else
  }, // end of rootElement getter

  /**
   * Returns application main window.
   */
  get mainWindow() {
    if (window.opener == null || window.opener == self) {
      return window.top;
    } // end of if
    
    else {
      return window.opener.top;
    } // end of else
  }, // end of mainWindow getter

  /**
   * Returns application session.
   */
  get session() {
    var mainWin = this.mainWindow;
    
    return mainWin.session;
  }, // end of session getter

  /**
   * Get message bundle.
   */
  get messageBundle() {
    var messageBundle;

    if ((messageBundle = this.viewDoc.getElementById("messages.bundle"))) {
      return messageBundle;
    } // end of if
    
    else {
      alert("{iPage} [Action] getMessageBundle -> No bundle named \"messages.bundle\"" +
	    "cannot be found");
      
      return null;
    } // end of else
  }, // end of messageBundle getter

  /**
   * Get error bundle.
   */
  get errorBundle() {
    var errorBundle;

    if ((errorBundle = this.viewDoc.getElementById("errors.bundle"))) {
      return errorBundle;
    } // end of if
    
    else {
      alert("{iPage} [Action] getErrorBundle -> No bundle named \"errors.bundle\"" +
	    "cannot be found");
      
      return null;
    } // end of else
  }, // end of errorBundle getter

  // ---

  /**
   * Make current window shake to indicate there is an error.
   */
  shakeError: function shakeError() {
    var n = 10;
    
    for (var i = 25; i > 0; i--) {
      for (var j = n; j > 0; j--) {
	self.moveBy(i,0);
	self.moveBy(-i,0);
      } // end of for
    } // end of for
  }, // end of shakeError

  /**
   * Disable specified command set.
   *
   * @param cmdSetName
   */
  disableCommandSet: function disableCommandSet(cmdSetName) {
    var cmdSetElmt = document.getElementById(cmdSetName);

    if (cmdSetElmt) {
      var cmdElmts = cmdSetElmt.getElementsByTagName("command");
      
      for (var i = 0; i < cmdElmts.length; i++) {
	cmdElmts[i].setAttribute("disabled", "true");
      } // end of for
    } // end of if

    else {
      /*
      logger.logStringMessage("{iPage} [Action] disableCommandSet -> " +
			      "No command set named \"" + cmdSetName + "\"");
      */
    } // end of else
  }, // end of disableCommandSet

  /**
   * Enable specified command set.
   *
   * @param cmdSetName
   */
  enableCommandSet: function enableCommandSet(cmdSetName) {
    var cmdSetElmt = document.getElementById(cmdSetName);

    if (cmdSetElmt) {
      var cmdElmts = cmdSetElmt.getElementsByTagName("command");
      
      for (var i = 0; i < cmdElmts.length; i++) {
	cmdElmts[i].setAttribute("disabled", "false");
      } // end of for
    } // end of if

    else {
      /*
      logger.logStringMessage("{iPage} [Action] enableCommandSet -> " +
			      "No command set named \"" + cmdSetName + "\"");
      */
    } // end of else
  }, // end of enableCommandSet

  /**
   * Disable specified command.
   *
   * @param cmdKey
   */
  disableCommand: function disableCommand(cmdKey) {
    if (this.viewDoc != null) {
      var cmdElmt = this.viewDoc.getElementById(cmdKey);

      if (cmdElmt != null) {
	cmdElmt.setAttribute("disabled", "true");
      } // end of if
    } // end of if
  }, // end of disableCommand

  /**
   * Enable specified command.
   *
   * @param cmdKey
   */
  enableCommand: function enableCommand(cmdKey) {
    if (this.viewDoc != null) {
      var cmdElmt = this.viewDoc.getElementById(cmdKey);

      if (cmdElmt != null) {
	cmdElmt.setAttribute("disabled", "false");
      } // end of if
    } // end of if
  }, // end of enableCommand
  
  /**
   * Tell UI that specified command is currently processed.
   *
   * @param cmdKey
   */
  setProcessingCommand: function setProcessingCommand(cmdKey) {
    if (this.viewDoc != null) {
      var rootElmt = this.rootElement;

      rootElmt.setAttribute("wait-cursor", "true");
      
      this.pendingCommands[cmdKey] = "pending";

      this.disableCommand(cmdKey);
    } // end of if
  }, // end of setProcessingCommand
  
  /**
   * Tell UI that specified command has been processed.
   *
   * @param cmdKey
   */
  setProcessedCommand: function setProcessedCommand(cmdKey) {
    if (this.viewDoc != null) {
      var rootElmt = this.rootElement;

      rootElmt.setAttribute("wait-cursor", "false");
      
      delete this.pendingCommands[cmdKey];

      this.enableCommand(cmdKey);
    } // end of if
  }, // end of setProcessedCommand

  /**
   * Fire 'oncommand' event for specified command.
   *
   * @param cmdKey Key for command to be fired.
   */
  fireCommand: function fireCommand(cmdKey) {
    var cmdElmt = this.viewDoc.getElementById(cmdKey);

    if (!cmdElmt.getAttribute("disabled") || 
	cmdElmt.getAttribute("disabled") == "false") {

      cmdElmt.doCommand();
    } // end of if
  }, // end of fireCommand

  /**
   * Clean all pending commands
   */
  clearPendingCommands: function clearPendingCommands() {
    var rootElmt = this.rootElement;
    
    rootElmt.style.cursor = "auto";
    
    // ---
    
    for (cmdKey in this.pendingCommands) {
      var cmdElmt = this.viewDoc.getElementById(cmdKey);

      if (cmdElmt) {
	cmdElmt.setAttribute("disabled", false);
      } // end of if

      // ---
      
      delete this.pendingCommands[cmdKey];    
    } // end of for
  }, // end of clearPendingCommands

  /**
   * Returns localized error corresponding to specified key.
   *
   * @param params Error object as error { key: "key"(, values: ["value1", "value2", ...]) }
   */
  actionError: function actionError(error) {
    if ((typeof error.values) == "undefined" ||
	error.values == null ||
	error.values.length == 0) {
      
      return this.errorBundle.
        getString(error.key);
      
    } // end of else if
    
    else { // argument.length > 1
      return this.errorBundle.
        getFormattedString(error.key, error.values);
      
    } // end of else
  }, // end of actionError

  /**
   * Default implementation for error handler.
   *
   * @param error Error to be treated.
   */
  onError: function onError(error) {
    var idx = -1;

    if ((typeof error.message) != "undefined" &&
	error.message != null &&
	(typeof error.message) == "string" &&
	(idx = error.message.
	 indexOf(ERROR_KEY_TOKEN)) != -1) {
      
      var errorKey = error.message.
	substring(idx + ERROR_KEY_TOKEN.length);

      if ((idx = errorKey.indexOf(":")) != -1) {
	errorKey = errorKey.substring(0, idx);
      } // end of if
      
      this.showErrorDialog({key: errorKey});
    } // end of if
    
    else {
      var errorObj = {
	key: "unknown", 
	values: [error.message]
      };
      
      this.showErrorDialog(errorObj);
    } // end of else
    
    // ---
    
    this.clearPendingCommands();
  }, // end of onError
  
  /**
   * Handler for field changed event.
   *
   * @param event Field change event.
   * @param cmdKey Name of command depending on changed field.
   */
  onFieldChanged: function onFieldChanged(event, cmdKey) {
    var errors = this.form.validate(cmdKey);
    var cmdElmt = this.viewDoc.getElementById(cmdKey);
    
    if (errors == null) {
      cmdElmt.setAttribute("disabled", false);
    } // end of if
    
    else {
      cmdElmt.setAttribute("disabled", true);
    } // end of else
  }, // end of onFieldChanged

  /**
   * Show error dialog featuring specified error.
   *
   * @param error1 Error object as error 
   * {key: "error.key"(, values: ["value1", "value2", ...])}
   * @param error2, ...
   */
  showErrorDialog: function showErrorDialog() {
    var errors = arguments[0];
    
    if (errors != null) {
      window.openDialog("chrome://ipage/content/error-dialog.xul",
			"error",
			"chrome,centerscreen,modal",
			errors);
      
    } // end of if
  }, // end of showErrorDialog

  /**
   * Forward user to another xul file.
   *
   * @param chromeUrl Chrome url for next xul file.
   */
  forwardTo: function forwardTo(chromeUrl) {
    self.location.href = chromeUrl;
  } // end of forwardTo
}; // end of Action prototype
