// import 'chrome://ipage/content/ws-creation-listener.js'
// import 'chrome://ipage/content/ws-delegate.js'
// import 'chrome://ipage/content/oop.js'

// --- Constants ---

/**
 * Url to get webservice definition.
 */
const PAGE_LIBRARY_WSDL_URL = "http://ipage-test.corsaire.fr:52080/nzkws/wsdl/PageLibraryService.wsdl";

/**
 * Name of webservice binding in WSDL file.
 */
const PAGE_LIBRARY_WSDL_BINDING = "PageLibraryPort";

/**
 * Logger
 */
var logger = Components.classes["@mozilla.org/consoleservice;1"].
  getService(Components.interfaces.nsIConsoleService);

/**
 * Page library webservice delegate.
 * className: PageLibraryWebserviceDelegate
 *
 * @author Cedric Chantepie (cchantepie@corsaire.fr)
 */
// --- Constructors ---

/**
 * Sole constructor.
 *
 * @param handler Webservice response handler
 */
function PageLibraryWebserviceDelegate(handler) {
  inherits(WebserviceDelegate, this);

  try {
    var factory = new WebServiceProxyFactory();
    var creationListener = new WebServiceCreationListener(this);

    this.handler = handler;

    factory.createProxyAsync(PAGE_LIBRARY_WSDL_URL, 
			     PAGE_LIBRARY_WSDL_BINDING, 
			     "",
			     true, 
			     creationListener);

  } // end of try

  catch (ex) {
    alert(ex);
  } // end of catch
} // end of <init>

PageLibraryWebserviceDelegate.prototype = {
  // extends WebserviceDelegate
  __proto__: WebserviceDelegate.prototype,

  // --- nsIClassInfo implementation ---

  /**
   * Return description of this class.
   */
  get classDescription() { 
    return "PageLibraryWebserviceDelegate"; 
  }, // end of classDescription
  
  // --- Service operations ---

  /**
   * Create a directory for page library. Id is generated.
   *
   * @param displayName Directory display name.
   * @param description Description for directory to be created.
   * @param priority Directory priority.
   * @param parentId Id of page library directory in which 
   * new directory should be created.
   * @return Created page library directory.
   */
  createDirectory: function createDirectory() {
    var displayName = arguments[0];
    var description = arguments[1];
    var priority = arguments[2];
    var parentId = arguments[3];

    try {
      this.delegatee.
	createDirectory(displayName,
			description,
			priority,
			parentId);

    } // end of try

    catch (e) {
      this.handler.onError(e);
    } // end of catch
  }, // end of createDirectory

  /**
   * Save directory in page bundle library,
   * or create it if it does not already exist.
   *
   * @param caller Principal for the caller of this functionality.
   * @param directory Directory to be saved.
   */
  saveDirectory: function saveDirectory() {
    var principal = arguments[0];
    var directory = arguments[1];

    try {
      this.delegatee.
	saveDirectory(principal,
		      directory);

    } // end of try

    catch (e) {
      this.handler.onError(e);
    } // end of catch
  }, // end of saveDirectory

  /**
   * Save page item in page library.
   *
   * @param caller Principal for the caller of this functionality.
   * @param locale Page item locale.
   * @param item Page library item.
   */
  savePageItem: function savePageItem() {
    var principal = arguments[0];
    var locale = arguments[1];
    var item = arguments[2];

    try {
      this.delegatee.
	savePageItem(principal,
		     locale,
		     item);

    } // end of try

    catch (e) {
      this.handler.onError(e);
    } // end of catch
  }, // end of savePageItem

  /**
   * Get all items contained in the directory corresponding
   * to given id.
   *
   * @param caller Principal for the caller of this functionality.
   * @param directoryId Id of directory to be listed.
   * @param locale Locale for retrieval.
   * @return Page library items.
   */
  getItems: function getItems() {
    var principal = arguments[0];
    var directoryId = arguments[1];
    var locale = arguments[2];
    
    try {
      this.delegatee.
	getItems(principal,
		 directoryId,
		 locale);
      
    } // end of try
  
    catch (e) {
      this.handler.onError(e);
    } // end of catch
  }, // end of getItems

  /**
   * Get directory according given id.
   *
   * @param caller Principal for the caller of this functionality.
   * @param id Id of directory.
   * @return Directory from page library corresponding to given id.
   */
  getDirectory: function getDirectory() {
    var principal = arguments[0];
    var id = arguments[1];

    try {
      this.delegatee.
	getItems(principal, id);

    } // end of try

    catch (e) {
      this.handler.onError(e);
    } // end of catch
  }, // end of

  /**
   * Get root directory.
   *
   * @param caller Principal for the caller of this functionality.
   * @return Root directory in page library.
   */
  getRoot: function getRoot() {
    var principal = arguments[0];
    
    try {
      this.delegatee.getRoot(principal);
    } // end of try
    
    catch (e) {
      this.handler.onError(e);
    } // end of catch
  } // end of getRoot

}; // end of PageLibraryWebserviceDelegate prototype
