// --- Global variables ---

/**
 */
var logger = Components.classes["@mozilla.org/consoleservice;1"].
  getService(Components.interfaces.nsIConsoleService);

/**
 */
var testAction = null;

/**
 */
// --- Constructors ---

/**
 * Constructor.
 *
 * @param viewDoc
 */
function TestAction(viewDoc) {
  inherits(Action, this);

  this.viewDoc = viewDoc;

  var treeView = new TreeView();

  this.listView = treeView;

  new LibraryTreeItem(this.listView,
		      1,
		      "Folder",
		      "XXX-XXX-XXX",
		      "Test1",
		      "Descr1",
		      null,
		      0);

  new LibraryTreeItem(this.listView,
		      0,
		      "Leaf",
		      "XXX-XXX-XXX",
		      "Test2",
		      "Descr2",
		      null,
		      0);

  this.viewDoc.getElementById("listView.tree").view = this.listView;

} // end of <init>

TestAction.prototype = {
  // extends Action
  __proto__: Action.prototype,

  // --- nsIClassInfo implementation ---

  /**
   * Return description of this class.
   */
  get classDescription() { 
    return "TestAction"; 
  }, // end of classDescription getter

  // --- Properties ---

  /**
   * List view (TreeView instance) if used, else null.
   */
  listView: null,

  // --- Getters ---

  /**
   * Returns currently selected item.
   */
  get activeSelectedItem() {
    if (this.listView != null) {
      var treeElmt = this.viewDoc.getElementById("listView.tree");
      var currentIndex = treeElmt.view.selection.currentIndex;
      
      return this.listView.getItemAtIndex(currentIndex);
    } // end of if

    else { // icon view
      return null;
    } // end of else
  }, // end of activeSelectedItem getter

  /**
   * Returns all selected items.
   */
  get selectedItems() {
    if (this.listView != null) {
      var selectedItems = new Array();
      var treeElmt = this.viewDoc.getElementById("listView.tree");
      var rangeCount = treeElmt.view.selection.getRangeCount();

      for (var i = 0; i < rangeCount; i++) {
	var start = { };
	var end = { };
	
	treeElmt.view.selection.getRangeAt(i, start, end);

	for (var j = start.value; j <= end.value; j++) {
	  selectedItems.push(this.listView.getItemAtIndex(j));
	} // end of for
      } // end of for

      return selectedItems;
    } // end of if

    else { // icon view

      return null;
    } // end of else
  }, // end of selectedItems getter

  /**
   * Change a property of active item.
   */
  changeProperty: function changeProperty() {
    var propertyName = this._changeProperty;

    logger.logStringMessage("changeProperty#propertyName=" + propertyName);

    if (this.listView) {
      var treeElmt = this.viewDoc.getElementById("listView.tree");
      var itemIndex = treeElmt.view.selection.currentIndex;

      logger.logStringMessage("changeProperty#itemIndex=" + itemIndex);

      treeElmt.setEditMode(itemIndex, {id: propertyName}, true);
    } // end of if

    delete this._changeProperty;
  }, // end of changeProperty

  /**
   * Fired when a cell content is accepted by user.
   */
  onCellAccept: function onCellAccept(itemIndex, column, initialValue, currentValue) {
  }, // end of onCellAccept

  /**
   * Handle event from selection change.
   *
   * @param event Selection change event.
   */
  onSelect: function onSelect(event) {
    var currentItem = this.activeSelectedItem;

    if (currentItem) {
      this.enableCommandSet("pagedir.cmdset");
    } // end of if

    else {
      this.disableCommandSet("pagedir.cmdset");
    } // end of else
  }, // end of onSelect

  /**
   * Handle context menu event.
   *
   * @param event Context menu event.
   */
  onContextMenu: function onContextMenu(event) {
    var treeBox = this.listView.treeBox
    var row = {};
    var col = {};
    var obj = {};

    treeBox.getCellAt(event.clientX,
		      event.clientY,
		      row,
		      col,
		      obj);

    if (col.value.id) {
      this._changeProperty = col.value.id;
    } // end of if
  } // end of onContextMenu

}; // end of TestAction prototype

// ---

/**
 */
function init() {
  testAction = new TestAction(document);

  var listViewElmt = document.getElementById("listView.tree");

  listViewElmt.onAccept = testAction.onCellAccept;
} // end of init
