/**
 * Library tree item.
 *
 * className: LibraryTreeItem
 */
// --- Constructors ---

/**
 * Constructor.
 *
 * @param parent Parent tree item.
 * @param type Internal type (LIBRARY_DIRECTORY | LIBRARY_ITEM)
 * @param humanType Human readable type (localized)
 * @param id Item id.
 * @param displayName
 * @param icon Item icon
 * @param priority Priority of item in library.
 */
function LibraryTreeItem(parent, 
			 type,
			 humanType,
			 id,
			 displayName,
			 description,
			 icon,
			 priority) {

  this._type = type;
  this.type = humanType;
  this.id = id;
  this._displayName = displayName;
  this._description = description;
  this.icon = icon;
  this.priority = priority;

  parent.appendItem(this);
} // end of <init>

LibraryTreeItem.prototype = {
  // extends TreeView
  __proto__: TreeView.prototype,

  // --- nsIClassInfo implementation ---

  /**
   */
  get classDescription() { 
    return "LibraryTreeItem"; 
  }, // end of classDescription getter

  // --- nsISupportsString implementation ---

  /**
   */
  get data() {
    return "[object " + this.classDescription + 
      "[displayName=" + this.displayName + "]]";

  }, // end of data getter

  /**
   * Get image for specified cell.
   *
   * @return Image to be displayed in specified cell.
   */
  getImageSrc: function getImageSrc(column) {
    if (column.index == 0) {
      return this.icon;
    } // end of if
  }, // end of getImageSrc

  /**
   * Returns whether specify column of this item
   * is editable. Base implementation always returns false.
   */
  isEditable: function isEditable(column) {
    return (column.id == "displayName" ||
	    column.id == "description");

  }, // end of isEditable

  // --- Properties ---

  /**
   * Id of item.
   */
  id: null,

  /**
   * Display name for this item.
   */
  _displayName: null,

  /**
   * Icon of item.
   */
  icon: null,

  /**
   * Item type [null].
   */
  type: null,

  /**
   * Internal type (LIBRARY_DIRECTORY | LIBRARY_LEAF) [-1].
   */
  _type: -1,

  /**
   * Item description text.
   */
  _description: null,

  /**
   * Item priority [-1]
   */
  priority: -1,

  // --- Property getters ---

  /**
   * Returns display name.
   */
  get displayName() {
    return this._displayName;
  }, // end of displayName getter

  /**
   * Set display name.
   */
  set displayName(displayName) {
    var oldValue = this._displayName;

    this._displayName = displayName;
  }, // end of displayName setter

  /**
   * Returns description.
   */
  get description() {
    return this._description;
  }, // end of description getter

  /**
   * Set description.
   */
  set description(description) {
    var oldValue = this._description;

    this._description = description
  }, // end of description setter

  // ---

  /**
   */
  isHidden: function isHidden(column) {
    var hidden = this[column.id + "Hidden"];

    if (typeof hidden == "undefined") {
      return false;
    } // end of if

    else {
      return hidden;
    } // end of else
  }, // end of isHidden

  /**
   */
  setHidden: function setHidden(column, hidden) {
    this[column.id + "Hidden"] = hidden;
  } // end of setHidden

} // end of LibraryTreeItem prototype
