// import 'chrome://ipage/content/arrays.js'
// import 'chrome://ipage/content/string.js'

/**
 * className: TreeView
 *
 * @author Cedric Chantepie (cchantepie@corsaire.fr)
 */
// --- Constructors ---

/**
 * Sole constructor.
 */
function TreeView() {
} // end of <init>

TreeView.prototype = {

  /* debugging */
  get wrappedJSObject() { return this; },

  // --- nsISupports implementation ---

  /**
   */
  QueryInterface: function QueryInterface(aIID) {
    if (Components.interfaces.nsITreeView.equals(aIID) ||
        Components.interfaces.nsIClassInfo.equals(aIID) ||
        Components.interfaces.nsISupportsString.equals(aIID) ||
        Components.interfaces.nsISupportsWeakReference.equals(aIID) ||
        Components.interfaces.nsISupports.equals(aIID)) {
	  
	  return this;
	} // end of if
    
    throw 0x80004002; // Components.results.NS_NOINTERFACE;
  }, // end of QueryInterface

  // --- nsIClassInfo implementation ---

  /**
   */
  getInterfaces: function getInterfaces(count) {
    count.value = 5;

    return [Components.interfaces.nsITreeView,
            Components.interfaces.nsIClassInfo,
            Components.interfaces.nsISupportsString,
            Components.interfaces.nsISupportsWeakReference,
            Components.interfaces.nsISupports];

  }, // end of getInterfaces

  /**
   */
  getHelperForLanguage: function getHelperForLanguage(language) { 
    return null;
  }, // end of getHelperForLanguage

  /**
   */
  get contractID() { 
    return null; 
  }, // end of contractID getter

  /**
   */
  get classDescription() { 
    return "TreeView"; 
  }, // end of classDescription getter
  
  /**
   */
  get classID() {
    return null; 
  }, // end of classID getter

  /**
   */
  get implementationLanguage() { 
    return Components.interfaces.nsIProgrammingLanguage.JAVASCRIPT; 
  }, // end of implementationLanguage getter
  
  /**
   */
  get flags() {
    return Components.interfaces.nsIClassInfo.MAIN_THREAD_ONLY | 
      Components.interfaces.nsIClassInfo.DOM_OBJECT; 

  }, // end of flags getter

  // --- nsISupportsString implementation ---

  /**
   */
  get data() {
    return "[object " + this.classDescription + "]"; 
  }, // end of data getter

  /**
   */
  toString: function toString() { 
    return this.data; 
  }, // end of toString

  // --- Properties ---
  
  /**
   */
  parentItem: null,

  /**
   */
  hasNext: false,

  /**
   */
  childItems: [],

  /**
   */
  subtreeItems: [],

  /**
   */
  open: false,

  /**
   */
  treeBox: null,

  /**
   * Indicated children has been loaded (if necessary).
   */
  subtreeLoaded: false,

  /**
   * Toggle state listeners.
   */
  toggleStateListeners: null,

  /**
   * [false]
   */
  sorted: false,

  // ---

  /**
   */
  get rowCount() {
    return this.subtreeItems.length;
  }, // end of rowCount getter

  /**
   */
  selection: null,

  // ---

  /**
   */
  getRowProperties: function getRowProperties(index, props) { 
  }, // end of getRowProperties

  /**
   */
  getCellProperties: function getCellProperties(index, column, props) { 
  }, // end of getCellProperties

  /**
   */
  getColumnProperties: function getColumnProperties(column, elem, props) { 
  }, // end of getColumnProperties

  /**
   */
  isContainer: function isContainer(index) {
    if (index in this.subtreeItems) {
      var subtree = this.subtreeItems[index];

      if (subtree._type == LIBRARY_DIRECTORY) {
	if (subtree.subtreeLoaded) {
	  var result = subtree.childItems.length; 
	  
	  return result;
	} // end of if
	
	else {
	  return true;
	} // end of else
      } // end of if

      else { // LIBRARY_LEAF
	return false;
      } // end of else
    } // end of if

    else {
      throw 0x8000FFFF; // Components.results.NS_ERROR_UNEXPECTED;
    } // end of else
  }, // end of isContainer

  /**
   */
  isContainerOpen: function isContainerOpen(index) {
    return this.subtreeItems[index].open;
  }, // end of isContainerOpen

  /**
   */
  isContainerEmpty: function isContainerEmpty(index) {
    var item = this.subtreeItems[index];

    return (item.subtreeLoaded && 
	    (!item.subtreeItems ||
	     item.subtreeItems.length == 0));

  }, // end of isContainerEmpty

  /**
   */
  isSeparator: function isSeparator(index) { 
    return false; 
  }, // end of isSeparator

  /**
   */
  isSorted: function isSorted() {
    return this.sorted;
  }, // end of isSorted

  // d&d not implemented yet!

  /**
   */
  canDropOn: function canDropOn(index) { 
    return this.isContainer(index);
  }, // end of canDropOn

  /**
   */
  canDropBeforeAfter: function canDropBeforeAfter(index, before) { 
    return true;
  }, // end of canDropBeforeAfter

  /**
   */
  drop: function drop(index, orientation) {
  }, // end of drop

  /**
   */
  getParentIndex: function getParentIndex(index) { 
    return this.getIndexOfItem(this.subtreeItems[index].parentItem); 
  }, // end of getParentIndex

  /**
   */
  hasNextSibling: function hasNextSibling(index, after) { 
    return this.subtreeItems[index].hasNext; 
  }, // end of hasNextSibling

  /**
   */
  getLevel: function getLevel(index) {
    if (index in this.subtreeItems) {
      var level = 0;

      for (var item = this.subtreeItems[index]; 
	   item.parentItem != this; 
	   ++level) {

        item = item.parentItem;
      } // end of if

      // ---

      return level;
    } // end of if

    else {
      throw 0x8000FFFF; // Components.results.NS_ERROR_UNEXPECTED;
    } // end of else
  }, // end of getLevel

  /**
   * Get image for specified cell.
   *
   * @param index Index of cell in specified column (x).
   * @param column Index of column in which is the cell (y).
   * @return Image to be displayed in specified cell.
   */
  getImageSrc: function getImageSrc(index, column) {
    var treeItem = this.subtreeItems[index];

    return treeItem.getImageSrc(column);
  }, // end of getImageSrc

  /**
   */
  getProgressMode: function getProgressMode(index, column) {
    return null;
  }, // end of getProcessMode

  /**
   */
  getCellValue: function getCellValue(index, column) {
    if (index in this.subtreeItems) {
      if (!this.subtreeItems[index].isHidden(column)) {
	//DEPRECATED:FF8:return this.subtreeItems[index][column];
	
	//FF8+
	return this.subtreeItems[index][column.id];
      } // end of if
      
      else {
	return null;
      } // end of else
    } // end of if

    else {
      throw 0x8000FFFF; // Components.results.NS_ERROR_UNEXPECTED;
    } // end of else
  }, // end of getCellValue

  /**
   */
  getCellText: function getCellText(index, column) {
    var value = this.getCellValue(index, column);

    return (value == null) ? "" : value;
  }, // end of getCellText

  /**
   */
  setTree: function setTree(treeBox) { 
    this.treeBox = treeBox; 
    
    if (!treeBox) {
      this.selection = null;
    } // end of if
  }, // end of setTree

  /**
   * Try to use function named "compare" + col.id (id with initial caps)
   * as comparator function to sort rows. If this function 
   * doesn't exist, nothing is done.
   */
  sortColumn: function sortColumn(col) {
    var colIdInitCap = col.id.charAt(0).toUpperCase() + col.id.substring(1);
    var comparator = this["compare" + colIdInitCap];

    if (comparator) {
      this.subtreeItems.sort(comparator);

      this.sorted = true;
    } // end of if
  }, // end of sortColumn

  /**
   */
  cycleHeader: function cycleHeader(col, elem) {
    if (!this.sorted) {
      this.sortColumn(col);
    } // end of if

    else {
      this.subtreeItems.reverse();
    } // end of else

    this.treeBox.invalidate();
  }, // end of cycleHeader
  
  /**
   * Handle change event for selection.
   */
  selectionChanged: function selectionChanged() {
  }, // end of selectionChanged

  /**
   */
  cycleCell: function cycleCell(index, column) {
  }, // end of cycleCell

  /**
   * Returns whether tree is editable
   */
  isEditable: function isEditable(index, column) {
    if (index in this.subtreeItems) {
      var item = this.subtreeItems[index];

      if (item.isEditable) {
	return item.isEditable(column);
      } // end of if

      else {
	alert("{iPage} [TreeView] isEditable -> item[" + index + 
	      "] does not implement isEditable(nsITreeColumn)");

	throw 0x8000FFFF; // Components.results.NS_ERROR_UNEXPECTED;
      } // end of else
    } // end of if

    else {
      throw 0x8000FFFF; // Components.results.NS_ERROR_UNEXPECTED;
    } // end of else
  }, // end of isEditable

  /**
   */
  performAction: function performAction(action) {
  }, // end of performAction
  
  /**
   */
  performActionOnCell: function performActionOnCell(action, index, column) {
  }, // end of performActionOnCell

  /**
   */
  toggleOpenState: function toggleOpenState(index) {
    this.subtreeItems[index].toggleState();
  }, // end of toggleOpenState

  // --- Utility ---

  /**
   */
  get childCount() {
    return this.childItems.length;
  }, // end of childCount getter

  /**
   */
  getIndexOfItem: function getIndexOfItem(item) {
    if (!item) {
      throw 0x80004003; // Components.results.NS_ERROR_NULL_POINTER;
    } // end of if

    else {
      var index = -1;

      while (item != this) {
	var parent = item.parentItem;

	if (!parent) {
	  throw 0x80004005; // Components.results.NS_ERROR_FAILURE;
	} // end of if

	// ---

	for (var i = 0; (tmp = parent.childItems[i]) != item; ++i) {
	  if (tmp.open) {
	    index += tmp.subtreeItems.length;
	  } // end of if
	} // end of for

	// ---

	index += i + 1;
	item = parent;
      } // end of while

      return index;
    } // end of else
  }, // end of getIndexOfItem
  
  /**
   */
  getIndexOfChild: function getIndexOfChild(item) {
    if (!item) {
      throw 0x80004003; // Components.results.NS_ERROR_NULL_POINTER;
    } // end of if
    
    // ---
    
    if (item.parentItem != this) {
      throw 0x80004005; // Components.results.NS_ERROR_FAILURE;
    } // end of if
    
    // ---

    for (var i = 0; i < this.childItems.length; ++i) {
      if (this.childItems.length[i] == item) {
	return i;
      } // end of if

      // ---

      throw 0x80004005; // Components.results.NS_ERROR_FAILURE;
    } // end of for

    // ---

    throw 0x80004005; // Components.results.NS_ERROR_FAILURE;
  }, // end of getIndexOfChild

  /**
   */
  getItemAtIndex: function getItemAtIndex(index) {
    index = parseInt(index) || 0;

    if (index < 0 || index >= this.subtreeItems.length) {
      throw 0x80004005; // Components.results.NS_ERROR_FAILURE;
    } // end of if

    else {
      return this.subtreeItems[index];
    } // end of else
  }, // end of getItemAtIndex

  /**
   */
  getChildAtIndex: function getChildAtIndex(index) {
    index = parseInt(index) || 0;

    if (index < 0 || index >= this.childItems.length) {
      throw 0x80004005; // Components.results.NS_ERROR_FAILURE;
    } // end of if

    else {
      return this.childItems[index];
    } // end of else
  }, // end of getChildAtIndex

  /**
   */
  selectItem: function selectItem(item) {
    for (var parent = item.parentItem; parent != this; parent = parent.parentItem) {
      if (!parent) {
	throw 0x80004005; // Components.results.NS_ERROR_FAILURE;
      } // end of if

      else if (!parent.open) {
	parent.toggleState();
      } // end of else if

      // ---
	
      var index = this.getIndexOfItem(item);

      this.selection.select(index);
      this.treeBox.ensureRowIsVisible(index);
    } // end of for
  }, // end of selectItem

  /**
   */
  invalidateRow: function invalidateRow() {
    var offset = -1;
    var parent;

    for (var item = this; parent = item.parentItem; item = parent) {
      offset += item.getOffset();

      // ---
	
      if (parent.treeBox) {
	parent.treeBox.invalidateRow(offset);
      } // end of if
	
      // ---
	
      if (!parent.open) {
	break;
      } // end of if
    } // end of for
  }, // end of invalidateRow
    
  /**
   */
  invalidatePrimaryCell: function invalidatePrimaryCell() {
    var offset = -1;
    var parent;

    for (var item = this; parent = item.parentItem; item = parent) {
      offset += item.getOffset();

      if (parent.treeBox) {
	//DEPRECATED:FF8:parent.treeBox.invalidatePrimaryCell(offset);

	//FF8+
	parent.treeBox.invalidateCell(offset, 
				      parent.treeBox.columns.getPrimaryColumn());

      } // end of if

      // ---

      if (!parent.open) {
	break;
      } // end of if
    } // end of for
  }, // end of invalidPrimaryCell

  /**
   */
  invalidateCell: function invalidateCell(column) {
    var offset = -1;
    var parent;

    for (var item = this; parent = item.parentItem; item = parent) {
      offset += item.getOffset();

      if (parent.treeBox) {
	parent.treeBox.invalidateCell(offset);
      } // end of if
	
      // ---

      if (!parent.open) {
	break;
      } // end of if
    } // end of for
  }, // end of invalidateCell

  /**
   * Add a toggle state listener.
   */
  addToggleStateListener: function addToggleStateListener(listener) {
    if (listener.onToggleState) {
      if (this.toggleStateListeners == null) {
	this.toggleStateListeners = [];
      } // end of if

      // ---

      var index = this.toggleStateListeners.length;

      this.toggleStateListeners[index] = listener;
    } // end of if

    else {
      throw new Error("[" + this.classDescription + "] fireToggleStateEvent " +
		      "-> Listener does not implements function onToggleState");

    } // end of else
  }, // end of addToggleStateListener

  /**
   * @param toggleStateEvent
   */
  fireToggleStateEvent: function fireToggleStateEvent() {
    var event = new Object();

    event.target = this;

    for (var i = 0; this.toggleStateListeners != null &&
	   i < this.toggleStateListeners.length; i++) {

      var listener = this.toggleStateListeners[i];

      listener.onToggleState(event);
    } // end of for
  }, // end of fireToggleStateEvent

  /**
   */
  toggleState: function toggleState() {
    this.open = !this.open;

    // must always be before length test
    this.fireToggleStateEvent();

    if (this.subtreeItems.length && this.parentItem) {
      if (this.open) {
	this.parentItem.itemExpanded(this.getOffset(), this.subtreeItems);
      } // end of if

      else {
	this.parentItem.itemCollapsed(this.getOffset(), this.subtreeItems.length);
      } // end of if
    } // end of if
  }, // end of toggleState

  /**
   */
  removeItem: function removeItem(item) {
    if (!item) {
      throw 0x80004003; // Components.results.NS_ERROR_NULL_POINTER;
    } // end of if

    // ---

    if (item.parentItem != this) {
      throw 0x80004005; // Components.results.NS_ERROR_FAILURE;
    } // end of if

    // ---

    var change = 1;

    if (item.open) {
      change += item.subtreeItems.length;
    } // end of if

    var offset = 0;
    var tmp;

    for (var i = 0; (tmp = this.childItems[i]) != item; ++i) {
      if (tmp.open) {
	offset += tmp.subtreeItems.length;
      } // end of if
    } // end of for

    // ---

    offset += i;
      
    this.childItems.splice(i, 1);
      
    if (i) {
      this.childItems[i - 1].hasNext = item.hasNext;
    } // end of if
      
    // ---
      
    item.hasNext = false;
      
    this.subtreeItems.splice(offset, change);
      
    if (this.treeBox) {
      this.treeBox.rowCountChanged(offset, -change);
    } // end of if
      
    // ---
      
    if (this.parentItem) {
      this.parentItem.itemCollapsed(offset + this.getOffset(), 
				    this.open ? change : 0, 
				    this.childItems.length);
	
    } // end of if
      
    // ---

    item.parentItem = null;
  }, // end of removeItem

  /**
   */
  appendItem: function appendItem(item) {
    this.insertItem(item, this.childItems.length);
  }, // end of appendItem

  /**
   */
  insertItem: function insertItem(item, index) {
    if (!item) {
      throw 0x80004003; // Components.results.NS_ERROR_NULL_POINTER;
    } // end of if

    // ---

    var length = this.childItems.length;
    index = parseInt(index) || 0;

    if (index < 0 || index > length) {
      throw 0x80004005; // Components.results.NS_ERROR_FAILURE;
    } // end of if

    // ---

    if (item.parentItem) {
      item.parentItem.removeItem(item);
    } // end of if

    // ---

    item.parentItem = this;
    var newItems = [item];
    var offset = index;

    if (!length) {
      this.childItems = newItems;
    } // end of if

    else {
      this.childItems.splice(index, 0, item);

      if (index == length) {
	this.childItems[length - 1].hasNext = true;
	offset = this.subtreeItems.length;
      } // end of if

      else {
	item.hasNext = true;

	for (var i = 0; i < index; ++i) {
	  if (this.childItems[i].open) {
	    offset += this.childItems[i].subtreeItems.length;
	  } // end of if
	} // end of for
      } // end of else
    } // end of else

    // ---

    if (item.open) {
      newItems = newItems.concat(item.subtreeItems);
    } // end of if

    // ---

    this.subtreeItems = this.subtreeItems.
    splice(0, offset).concat(newItems).
    concat(this.subtreeItems);

    // ---

    if (this.treeBox) {
      this.treeBox.rowCountChanged(offset, newItems.length);
    } // end of if

    // ---

    if (this.parentItem && (this.open || !length)) {
      this.parentItem.itemExpanded(offset + this.getOffset(), 
				   this.open ? newItems : [], length);

    } // end of if
  }, // end of insertItem


  /**
   */
  isOpen: function isOpened() {
    return this.open;
  }, // end of isOpen

  /* helper methods */

  /**
   */
  itemExpanded: function itemExpanded(offset, newItems, notwisty) {
    this.subtreeItems = this.subtreeItems.splice(0, offset).
      concat(newItems).concat(this.subtreeItems);

    if (this.treeBox) {
      this.treeBox.rowCountChanged(offset, newItems.length);

      if (offset && !notwisty) {
	//DEPRECATED:FF8:this.treeBox.invalidatePrimaryCell(offset - 1);

	//FF8+
	this.treeBox.invalidateCell(offset - 1, 
				    this.treeBox.columns.getPrimaryColumn());

      } // end of if
    } // end of if

    // ---

    if (this.open && this.parentItem) {
      this.parentItem.itemExpanded(offset + this.getOffset(), newItems);
    } // end of if
  }, // end of itemExpanded

  /**
   */
  itemCollapsed: function itemCollapsed(offset, change, notwisty) {
    this.subtreeItems.splice(offset, change);

    if (this.treeBox) {
      this.treeBox.rowCountChanged(offset, -change);

      if (offset && !notwisty) {
	//DEPRECATED:FF8:this.treeBox.invalidatePrimaryCell(offset - 1);

	//FF8+
	this.treeBox.invalidateCell(offset - 1, 
				    this.treeBox.columns.getPrimaryColumn());
      } // end of if
    } // end of if

    // ---

    if (this.open && this.parentItem) {
      this.parentItem.itemCollapsed(offset + this.getOffset(), change);
    } // end of if
  }, // end of itemCollapsed

  /**
   */
  getOffset: function getOffset() {
    var offset = 1;
    var tmp;

    for (var i = 0; (tmp = this.parentItem.childItems[i]) != this; ++i) {
      if (tmp.open) {
	offset += tmp.subtreeItems.length;
      } // end of if
    } // end of for

    // ---
	
    return offset + i;
  }, // end of getOffset

  /**
   * Clear tree items.
   *
   * @param subtree If true only subtree item will be cleared, 
   * not this item itself.
   */
  clear: function clear(subtree) {
    var item = null;
    var items = new Array();
    var dirs = new Array();

    dirs.push(this);

    if (!subtree) {
      items.push(this);
    } // end of if

    while (dirs.length > 0) {
      var dir = dirs.shift();

      for (var i = 0; i < dir.subtreeItems.length; i++) {
	item = dir.subtreeItems[i];

	if (item._type == LIBRARY_DIRECTORY &&
	    item.subtreeItems.length > 0) {
	  
	  dirs.push(item);
	} // end of if

	// ---

	items.push(item);
      } // end of for
    } // end of while

    items.reverse();

    while (items.length > 0) {
      item = items.shift();
      var parentItem = item.parentItem;

      parentItem.removeItem(item);
    } // end of while
  } // end of clear

}; // end of TreeView prototype
